const chai = require('chai');
const chaiHttp = require('chai-http');
const fs = require('fs');
const path = require('path');
const app = require('../index');
const expect = chai.expect;

chai.use(chaiHttp);

let authToken;
let adId;

describe('Ad Routes', () => {
    before(async () => {
        try {
            const loginResponse = await chai
                .request(app)
                .post('/user/login')
                .send({ username: 'hajar', password: 'hajar' });

            authToken = loginResponse.body.token;
        } catch (error) {
            console.error('Error obtaining authentication token:', error);
        }
    });

    describe('POST /ads', () => {
        it('should add a new ad', async () => {
            const response = await chai
                .request(app)
                .post('/ad')
                .set('Authorization', `Bearer ${authToken}`)
                .send({
                    title: 'Test Ad',
                    propertyType: 'À la vente',
                    publicationStatus: 'Publiée',
                    propertyStatus: 'Disponible',
                    description: 'Test description',
                    price: 80000,
                    availabilityDate: '2024-12-01',
                });
            expect(response.body).to.have.property('_id').to.be.a('string');
            adId = response.body._id;
        });

    const backupFolderPath = path.join(__dirname, '..', 'backup');

        it('should add a new ad with photos', async () => {
            const response = await chai
                .request(app)
                .post('/ad')
                .set('Authorization', `Bearer ${authToken}`)
                .attach('photos', path.join(backupFolderPath, 'img1.jpg'))
                .attach('photos', path.join(backupFolderPath, 'img2.jpg'))
                .field('title', 'Test Ad 2')
                .field('propertyType', 'À la vente')
                .field('publicationStatus', 'Publiée')
                .field('propertyStatus', 'Disponible')
                .field('description', 'teest.')
                .field('price', 80000)
                .field('availabilityDate', '2024-11-01');

            expect(response).to.have.status(200);
            expect(response.body).to.have.property('_id').to.be.a('string');
            adId = response.body._id;
        });
    });



    describe('GET /ads', () => {
        it('should get all ads', async () => {
            const response = await chai
                .request(app)
                .get('/ad')
                .set('Authorization', `Bearer ${authToken}`);

            expect(response).to.have.status(200);
            expect(response.body).to.be.an('array');
        });
    });

    // Cleanup after each test
    after(async () => {
        try {
            // Add logic to delete all ads created during the tests
            const allAdsResponse = await chai
                .request(app)
                .get('/ad')
                .set('Authorization', `Bearer ${authToken}`);

            if (allAdsResponse.body && allAdsResponse.body.length > 0) {
                const deletePromises = allAdsResponse.body.map(async (ad) => {
                    await chai
                        .request(app)
                        .delete(`/ad/${ad._id}`)
                        .set('Authorization', `Bearer ${authToken}`);
                });

                await Promise.all(deletePromises);
            }

            // Add logic to delete all photos uploaded during the tests
            const uploadedPhotosPath = path.join(__dirname, '..', 'uploads');
            if (fs.existsSync(uploadedPhotosPath)) {
                fs.readdirSync(uploadedPhotosPath).forEach((file) => {
                    const filePath = path.join(uploadedPhotosPath, file);
                    fs.unlinkSync(filePath);
                });
                fs.rmdirSync(uploadedPhotosPath);
            }
        } catch (error) {
            console.error('Error cleaning up after all tests:', error);
        }
    });
});
